@@ -498,7 +498,7 @@ static struct ast_manager_event_blob *local_message_to_ami(struct stasis_message
 		}
 
 		dest_uniqueid = ast_json_object_get(blob, "dest") == AST_UNREAL_OWNER ?
-				local_snapshot_one->uniqueid : local_snapshot_two->uniqueid;
+				local_snapshot_one->base->uniqueid : local_snapshot_two->base->uniqueid;
 
 		event = "LocalOptimizationBegin";
 		if (source_str) {
@@ -967,8 +967,7 @@ static struct ast_channel *local_request_with_stream_topology(const char *type,
 
 		stream = ast_stream_topology_get_stream(audio_filtered_topology, i);
 
-		if (ast_stream_get_type(stream) != AST_MEDIA_TYPE_AUDIO ||
-			ast_stream_get_state(stream) == AST_STREAM_STATE_REMOVED) {
+		if (ast_stream_get_type(stream) != AST_MEDIA_TYPE_AUDIO) {
 			continue;
 		}
 
