@@ -120,8 +120,6 @@
 							<enum name="ast_channel_hangup_request_type" />
 							<enum name="ast_channel_dtmf_begin_type" />
 							<enum name="ast_channel_dtmf_end_type" />
-							<enum name="ast_channel_flash_type" />
-							<enum name="ast_channel_wink_type" />
 							<enum name="ast_channel_hold_type" />
 							<enum name="ast_channel_unhold_type" />
 							<enum name="ast_channel_chanspy_start_type" />
@@ -251,8 +249,8 @@
  * The destructors of both have assertions regarding this to catch ref-counting
  * problems where a subscription or topic has had an extra ao2_cleanup().
  *
- * The \ref dispatch_exec_sync object is a transient object, which is posted to
- * a subscription's taskprocessor to send a message to the subscriber. They have
+ * The \ref dispatch object is a transient object, which is posted to a
+ * subscription's taskprocessor to send a message to the subscriber. They have
  * short life cycles, allocated on one thread, destroyed on another.
  *
  * During shutdown, or the deletion of a domain object, there are a flurry of
@@ -751,6 +749,7 @@ static void subscription_dtor(void *obj)
 /*!
  * \brief Invoke the subscription's callback.
  * \param sub Subscription to invoke.
+ * \param topic Topic message was published to.
  * \param message Message to send.
  */
 static void subscription_invoke(struct stasis_subscription *sub,
@@ -1433,8 +1432,8 @@ static void publish_msg(struct stasis_topic *topic,
 	struct stasis_message *message, struct stasis_subscription *sync_sub)
 {
 	size_t i;
-#ifdef AST_DEVMODE
 	unsigned int dispatched = 0;
+#ifdef AST_DEVMODE
 	int message_type_id = stasis_message_type_id(stasis_message_type(message));
 	struct stasis_message_type_statistics *statistics;
 	struct timeval start;
@@ -1484,10 +1483,8 @@ static void publish_msg(struct stasis_topic *topic,
 		struct stasis_subscription *sub = AST_VECTOR_GET(&topic->subscribers, i);
 
 		ast_assert(sub != NULL);
-#ifdef AST_DEVMODE
-		dispatched +=
-#endif
-			dispatch_message(sub, message, (sub == sync_sub));
+
+		dispatched += dispatch_message(sub, message, (sub == sync_sub));
 	}
 	ao2_unlock(topic);
 
@@ -2298,7 +2295,7 @@ int stasis_message_type_declined(const char *name)
 	ao2_cleanup(name_in_declined);
 	ao2_ref(cfg, -1);
 	if (res) {
-		ast_debug(4, "Declining to allocate Stasis message type '%s' due to configuration\n", name);
+		ast_log(LOG_NOTICE, "Declining to allocate Stasis message type '%s' due to configuration\n", name);
 	}
 	return res;
 }
@@ -2420,6 +2417,7 @@ static char *stasis_show_topic(struct ast_cli_entry *e, int cmd, struct ast_cli_
 {
 	struct stasis_topic *topic;
 	char print_time[32];
+	int i;
 
 	switch (cmd) {
 	case CLI_INIT:
@@ -2453,6 +2451,21 @@ static char *stasis_show_topic(struct ast_cli_entry *e, int cmd, struct ast_cli_
 	ast_format_duration_hh_mm_ss(ast_tvnow().tv_sec - topic->creationtime->tv_sec, print_time, sizeof(print_time));
 	ast_cli(a->fd, "Duration time: %s\n", print_time);
 
+	ao2_lock(topic);
+	ast_cli(a->fd, "\nSubscribers:\n");
+	for (i = 0; i < AST_VECTOR_SIZE(&topic->subscribers); i++) {
+		struct stasis_subscription *subscription_tmp = AST_VECTOR_GET(&topic->subscribers, i);
+		ast_cli(a->fd, "  UniqueID: %s, Topic: %s, Detail: %s\n",
+				subscription_tmp->uniqueid, subscription_tmp->topic->name, subscription_tmp->topic->detail);
+	}
+
+	ast_cli(a->fd, "\nForwarded topics:\n");
+	for (i = 0; i < AST_VECTOR_SIZE(&topic->upstream_topics); i++) {
+		struct stasis_topic *topic_tmp = AST_VECTOR_GET(&topic->upstream_topics, i);
+		ast_cli(a->fd, "  Topic: %s, Detail: %s\n", topic_tmp->name, topic_tmp->detail);
+	}
+	ao2_unlock(topic);
+
 	ao2_ref(topic, -1);
 
 	return CLI_SUCCESS;
@@ -3176,31 +3189,3 @@ int stasis_init(void)
 
 	return 0;
 }
-
-#ifdef AST_DEVMODE
-#undef stasis_subscribe
-struct stasis_subscription *stasis_subscribe(
-	struct stasis_topic *topic,
-	stasis_subscription_cb callback,
-	void *data);
-#undef stasis_subscribe_pool
-struct stasis_subscription *stasis_subscribe_pool(
-	struct stasis_topic *topic,
-	stasis_subscription_cb callback,
-	void *data);
-#endif
-struct stasis_subscription *stasis_subscribe(
-	struct stasis_topic *topic,
-	stasis_subscription_cb callback,
-	void *data)
-{
-	return internal_stasis_subscribe(topic, callback, data, 1, 0, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
-
-struct stasis_subscription *stasis_subscribe_pool(
-	struct stasis_topic *topic,
-	stasis_subscription_cb callback,
-	void *data)
-{
-	return internal_stasis_subscribe(topic, callback, data, 1, 1, __FILE__, __LINE__, __PRETTY_FUNCTION__);
-}
