@@ -36,7 +36,6 @@
 #include "asterisk/bridge.h"
 #include "asterisk/translate.h"
 #include "asterisk/stasis.h"
-#include "asterisk/stasis_cache_pattern.h"
 #include "asterisk/stasis_channels.h"
 #include "asterisk/dial.h"
 #include "asterisk/linkedlists.h"
@@ -117,53 +116,23 @@
 
 #define NUM_MULTI_CHANNEL_BLOB_BUCKETS 7
 
-static struct stasis_cp_all *channel_cache_all;
-static struct stasis_cache *channel_cache_by_name;
-static struct stasis_caching_topic *channel_by_name_topic;
+static struct stasis_topic *channel_topic_all;
+static struct ao2_container *channel_cache;
+static struct ao2_container *channel_cache_by_name;
 
-struct stasis_cp_all *ast_channel_cache_all(void)
+struct ao2_container *ast_channel_cache_all(void)
 {
-	return channel_cache_all;
-}
-
-struct stasis_cache *ast_channel_cache(void)
-{
-	return stasis_cp_all_cache(channel_cache_all);
+	return ao2_bump(channel_cache);
 }
 
 struct stasis_topic *ast_channel_topic_all(void)
 {
-	return stasis_cp_all_topic(channel_cache_all);
+	return channel_topic_all;
 }
 
-struct stasis_topic *ast_channel_topic_all_cached(void)
+struct ao2_container *ast_channel_cache_by_name(void)
 {
-	return stasis_cp_all_topic_cached(channel_cache_all);
-}
-
-struct stasis_cache *ast_channel_cache_by_name(void)
-{
-	return channel_cache_by_name;
-}
-
-static const char *channel_snapshot_get_id(struct stasis_message *message)
-{
-	struct ast_channel_snapshot *snapshot;
-	if (ast_channel_snapshot_type() != stasis_message_type(message)) {
-		return NULL;
-	}
-	snapshot = stasis_message_data(message);
-	return snapshot->uniqueid;
-}
-
-static const char *channel_snapshot_get_name(struct stasis_message *message)
-{
-	struct ast_channel_snapshot *snapshot;
-	if (ast_channel_snapshot_type() != stasis_message_type(message)) {
-		return NULL;
-	}
-	snapshot = stasis_message_data(message);
-	return snapshot->name;
+	return ao2_bump(channel_cache_by_name);
 }
 
 /*!
@@ -180,7 +149,7 @@ static int channel_snapshot_hash_cb(const void *obj, const int flags)
 		key = obj;
 		break;
 	case OBJ_SEARCH_OBJECT:
-		key = object->name;
+		key = object->base->name;
 		break;
 	default:
 		ast_assert(0);
@@ -202,12 +171,65 @@ static int channel_snapshot_cmp_cb(void *obj, void *arg, int flags)
 
 	switch (flags & OBJ_SEARCH_MASK) {
 	case OBJ_SEARCH_OBJECT:
-		right_key = object_right->name;
+		right_key = object_right->base->name;
 	case OBJ_SEARCH_KEY:
-		cmp = strcasecmp(object_left->name, right_key);
+		cmp = strcasecmp(object_left->base->name, right_key);
 		break;
 	case OBJ_SEARCH_PARTIAL_KEY:
-		cmp = strncasecmp(object_left->name, right_key, strlen(right_key));
+		cmp = strncasecmp(object_left->base->name, right_key, strlen(right_key));
+		break;
+	default:
+		cmp = 0;
+		break;
+	}
+	if (cmp) {
+		return 0;
+	}
+	return CMP_MATCH;
+}
+
+/*!
+ * \internal
+ * \brief Hash function (using uniqueid) for \ref ast_channel_snapshot objects
+ */
+static int channel_snapshot_uniqueid_hash_cb(const void *obj, const int flags)
+{
+	const struct ast_channel_snapshot *object = obj;
+	const char *key;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_KEY:
+		key = obj;
+		break;
+	case OBJ_SEARCH_OBJECT:
+		key = object->base->uniqueid;
+		break;
+	default:
+		ast_assert(0);
+		return 0;
+	}
+	return ast_str_case_hash(key);
+}
+
+/*!
+ * \internal
+ * \brief Comparison function (using uniqueid) for \ref ast_channel_snapshot objects
+ */
+static int channel_snapshot_uniqueid_cmp_cb(void *obj, void *arg, int flags)
+{
+	const struct ast_channel_snapshot *object_left = obj;
+	const struct ast_channel_snapshot *object_right = arg;
+	const char *right_key = arg;
+	int cmp;
+
+	switch (flags & OBJ_SEARCH_MASK) {
+	case OBJ_SEARCH_OBJECT:
+		right_key = object_right->base->uniqueid;
+	case OBJ_SEARCH_KEY:
+		cmp = strcasecmp(object_left->base->uniqueid, right_key);
+		break;
+	case OBJ_SEARCH_PARTIAL_KEY:
+		cmp = strncasecmp(object_left->base->uniqueid, right_key, strlen(right_key));
 		break;
 	default:
 		cmp = 0;
@@ -223,36 +245,180 @@ static void channel_snapshot_dtor(void *obj)
 {
 	struct ast_channel_snapshot *snapshot = obj;
 
-	ast_string_field_free_memory(snapshot);
+	ao2_cleanup(snapshot->base);
+	ao2_cleanup(snapshot->peer);
+	ao2_cleanup(snapshot->caller);
+	ao2_cleanup(snapshot->connected);
+	ao2_cleanup(snapshot->bridge);
+	ao2_cleanup(snapshot->dialplan);
+	ao2_cleanup(snapshot->hangup);
 	ao2_cleanup(snapshot->manager_vars);
 	ao2_cleanup(snapshot->ari_vars);
 }
 
-struct ast_channel_snapshot *ast_channel_snapshot_create(struct ast_channel *chan)
+static void channel_snapshot_base_dtor(void *obj)
 {
-	struct ast_channel_snapshot *snapshot;
-	struct ast_bridge *bridge;
+	struct ast_channel_snapshot_base *snapshot = obj;
 
-	/* no snapshots for dummy channels */
-	if (!ast_channel_tech(chan)) {
+	ast_string_field_free_memory(snapshot);
+}
+
+static struct ast_channel_snapshot_base *channel_snapshot_base_create(struct ast_channel *chan)
+{
+	struct ast_channel_snapshot_base *snapshot;
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot), channel_snapshot_base_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
 		return NULL;
 	}
 
-	snapshot = ao2_alloc_options(sizeof(*snapshot), channel_snapshot_dtor,
-		AO2_ALLOC_OPT_LOCK_NOLOCK);
-	if (!snapshot || ast_string_field_init(snapshot, 1024)) {
-		ao2_cleanup(snapshot);
+	if (ast_string_field_init(snapshot, 256)) {
+		ao2_ref(snapshot, -1);
 		return NULL;
 	}
 
 	ast_string_field_set(snapshot, name, ast_channel_name(chan));
 	ast_string_field_set(snapshot, type, ast_channel_tech(chan)->type);
 	ast_string_field_set(snapshot, accountcode, ast_channel_accountcode(chan));
-	ast_string_field_set(snapshot, peeraccount, ast_channel_peeraccount(chan));
 	ast_string_field_set(snapshot, userfield, ast_channel_userfield(chan));
 	ast_string_field_set(snapshot, uniqueid, ast_channel_uniqueid(chan));
-	ast_string_field_set(snapshot, linkedid, ast_channel_linkedid(chan));
-	ast_string_field_set(snapshot, hangupsource, ast_channel_hangupsource(chan));
+	ast_string_field_set(snapshot, language, ast_channel_language(chan));
+
+	snapshot->creationtime = ast_channel_creationtime(chan);
+	snapshot->tech_properties = ast_channel_tech(chan)->properties;
+
+	return snapshot;
+}
+
+static struct ast_channel_snapshot_peer *channel_snapshot_peer_create(struct ast_channel *chan)
+{
+	const char *linkedid = S_OR(ast_channel_linkedid(chan), "");
+	const char *peeraccount = S_OR(ast_channel_peeraccount(chan), "");
+	size_t linkedid_len = strlen(linkedid) + 1;
+	size_t peeraccount_len = strlen(peeraccount) + 1;
+	struct ast_channel_snapshot_peer *snapshot;
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot) + linkedid_len + peeraccount_len, NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
+		return NULL;
+	}
+
+	strcpy(snapshot->account, peeraccount); /* Safe */
+	snapshot->linkedid = snapshot->account + peeraccount_len;
+	ast_copy_string(snapshot->linkedid, linkedid, linkedid_len); /* Safe */
+
+	return snapshot;
+}
+
+static void channel_snapshot_caller_dtor(void *obj)
+{
+	struct ast_channel_snapshot_caller *snapshot = obj;
+
+	ast_string_field_free_memory(snapshot);
+}
+
+static struct ast_channel_snapshot_caller *channel_snapshot_caller_create(struct ast_channel *chan)
+{
+	struct ast_channel_snapshot_caller *snapshot;
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot), channel_snapshot_caller_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
+		return NULL;
+	}
+
+	if (ast_string_field_init(snapshot, 256)) {
+		ao2_ref(snapshot, -1);
+		return NULL;
+	}
+
+	ast_string_field_set(snapshot, name,
+		S_COR(ast_channel_caller(chan)->id.name.valid, ast_channel_caller(chan)->id.name.str, ""));
+	ast_string_field_set(snapshot, number,
+		S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, ""));
+	ast_string_field_set(snapshot, subaddr,
+		S_COR(ast_channel_caller(chan)->id.subaddress.valid, ast_channel_caller(chan)->id.subaddress.str, ""));
+	ast_string_field_set(snapshot, ani,
+		S_COR(ast_channel_caller(chan)->ani.number.valid, ast_channel_caller(chan)->ani.number.str, ""));
+
+	ast_string_field_set(snapshot, rdnis,
+		S_COR(ast_channel_redirecting(chan)->from.number.valid, ast_channel_redirecting(chan)->from.number.str, ""));
+
+	ast_string_field_set(snapshot, dnid,
+		S_OR(ast_channel_dialed(chan)->number.str, ""));
+	ast_string_field_set(snapshot, dialed_subaddr,
+		S_COR(ast_channel_dialed(chan)->subaddress.valid, ast_channel_dialed(chan)->subaddress.str, ""));
+
+	snapshot->pres = ast_party_id_presentation(&ast_channel_caller(chan)->id);
+
+	return snapshot;
+}
+
+static struct ast_channel_snapshot_connected *channel_snapshot_connected_create(struct ast_channel *chan)
+{
+	const char *name = S_COR(ast_channel_connected(chan)->id.name.valid, ast_channel_connected(chan)->id.name.str, "");
+	const char *number = S_COR(ast_channel_connected(chan)->id.number.valid, ast_channel_connected(chan)->id.number.str, "");
+	size_t name_len = strlen(name) + 1;
+	size_t number_len = strlen(number) + 1;
+	struct ast_channel_snapshot_connected *snapshot;
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot) + name_len + number_len, NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
+		return NULL;
+	}
+
+	strcpy(snapshot->name, name); /* Safe */
+	snapshot->number = snapshot->name + name_len;
+	ast_copy_string(snapshot->number, number, number_len); /* Safe */
+
+	return snapshot;
+}
+
+static struct ast_channel_snapshot_bridge *channel_snapshot_bridge_create(struct ast_channel *chan)
+{
+	const char *uniqueid = "";
+	struct ast_bridge *bridge;
+	struct ast_channel_snapshot_bridge *snapshot;
+
+	bridge = ast_channel_get_bridge(chan);
+	if (bridge && !ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_INVISIBLE)) {
+		uniqueid = bridge->uniqueid;
+	}
+	ao2_cleanup(bridge);
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot) + strlen(uniqueid) + 1, NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
+		return NULL;
+	}
+
+	strcpy(snapshot->id, uniqueid); /* Safe */
+
+	return snapshot;
+}
+
+static void channel_snapshot_dialplan_dtor(void *obj)
+{
+	struct ast_channel_snapshot_dialplan *snapshot = obj;
+
+	ast_string_field_free_memory(snapshot);
+}
+
+static struct ast_channel_snapshot_dialplan *channel_snapshot_dialplan_create(struct ast_channel *chan)
+{
+	struct ast_channel_snapshot_dialplan *snapshot;
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot), channel_snapshot_dialplan_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
+		return NULL;
+	}
+
+	if (ast_string_field_init(snapshot, 256)) {
+		ao2_ref(snapshot, -1);
+		return NULL;
+	}
+
 	if (ast_channel_appl(chan)) {
 		ast_string_field_set(snapshot, appl, ast_channel_appl(chan));
 	}
@@ -261,54 +427,162 @@ struct ast_channel_snapshot *ast_channel_snapshot_create(struct ast_channel *cha
 	}
 	ast_string_field_set(snapshot, context, ast_channel_context(chan));
 	ast_string_field_set(snapshot, exten, ast_channel_exten(chan));
-
-	ast_string_field_set(snapshot, caller_name,
-		S_COR(ast_channel_caller(chan)->id.name.valid, ast_channel_caller(chan)->id.name.str, ""));
-	ast_string_field_set(snapshot, caller_number,
-		S_COR(ast_channel_caller(chan)->id.number.valid, ast_channel_caller(chan)->id.number.str, ""));
-	ast_string_field_set(snapshot, caller_subaddr,
-		S_COR(ast_channel_caller(chan)->id.subaddress.valid, ast_channel_caller(chan)->id.subaddress.str, ""));
-	ast_string_field_set(snapshot, caller_ani,
-		S_COR(ast_channel_caller(chan)->ani.number.valid, ast_channel_caller(chan)->ani.number.str, ""));
-
-	ast_string_field_set(snapshot, caller_rdnis,
-		S_COR(ast_channel_redirecting(chan)->from.number.valid, ast_channel_redirecting(chan)->from.number.str, ""));
-
-	ast_string_field_set(snapshot, caller_dnid,
-		S_OR(ast_channel_dialed(chan)->number.str, ""));
-	ast_string_field_set(snapshot, dialed_subaddr,
-		S_COR(ast_channel_dialed(chan)->subaddress.valid, ast_channel_dialed(chan)->subaddress.str, ""));
-
-	ast_string_field_set(snapshot, connected_name,
-		S_COR(ast_channel_connected(chan)->id.name.valid, ast_channel_connected(chan)->id.name.str, ""));
-	ast_string_field_set(snapshot, connected_number,
-		S_COR(ast_channel_connected(chan)->id.number.valid, ast_channel_connected(chan)->id.number.str, ""));
-
-	ast_string_field_set(snapshot, language, ast_channel_language(chan));
-
-	if ((bridge = ast_channel_get_bridge(chan))) {
-		if (!ast_test_flag(&bridge->feature_flags, AST_BRIDGE_FLAG_INVISIBLE)) {
-			ast_string_field_set(snapshot, bridgeid, bridge->uniqueid);
-		}
-		ao2_cleanup(bridge);
-	}
-
-	snapshot->creationtime = ast_channel_creationtime(chan);
-	snapshot->state = ast_channel_state(chan);
 	snapshot->priority = ast_channel_priority(chan);
-	snapshot->amaflags = ast_channel_amaflags(chan);
-	snapshot->hangupcause = ast_channel_hangupcause(chan);
-	ast_copy_flags(&snapshot->flags, ast_channel_flags(chan), 0xFFFFFFFF);
-	snapshot->caller_pres = ast_party_id_presentation(&ast_channel_caller(chan)->id);
-	ast_set_flag(&snapshot->softhangup_flags, ast_channel_softhangup_internal_flag(chan));
-
-	snapshot->manager_vars = ast_channel_get_manager_vars(chan);
-	snapshot->ari_vars = ast_channel_get_ari_vars(chan);
-	snapshot->tech_properties = ast_channel_tech(chan)->properties;
 
 	return snapshot;
 }
 
+static struct ast_channel_snapshot_hangup *channel_snapshot_hangup_create(struct ast_channel *chan)
+{
+	const char *hangupsource = S_OR(ast_channel_hangupsource(chan), "");
+	struct ast_channel_snapshot_hangup *snapshot;
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot) + strlen(hangupsource) + 1, NULL, AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
+		return NULL;
+	}
+
+	snapshot->cause = ast_channel_hangupcause(chan);
+	strcpy(snapshot->source, hangupsource); /* Safe */
+
+	return snapshot;
+}
+
+struct ast_channel_snapshot *ast_channel_snapshot_create(struct ast_channel *chan)
+{
+	struct ast_channel_snapshot *old_snapshot;
+	struct ast_channel_snapshot *snapshot;
+
+	/* no snapshots for dummy channels */
+	if (!ast_channel_tech(chan)) {
+		return NULL;
+	}
+
+	snapshot = ao2_alloc_options(sizeof(*snapshot), channel_snapshot_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!snapshot) {
+		return NULL;
+	}
+
+	old_snapshot = ast_channel_snapshot(chan);
+
+	/* Channels automatically have all segments invalidated on them initially so a check for an old
+	 * snapshot existing before usage is not done here, as it can not happen. If the stored snapshot
+	 * on the channel is updated as a result of this then all segments marked as invalidated will be
+	 * cleared.
+	 */
+	if (ast_test_flag(ast_channel_snapshot_segment_flags(chan), AST_CHANNEL_SNAPSHOT_INVALIDATE_BASE)) {
+		/* The base information has changed so update our snapshot */
+		snapshot->base = channel_snapshot_base_create(chan);
+		if (!snapshot->base) {
+			ao2_ref(snapshot, -1);
+			return NULL;
+		}
+	} else {
+		snapshot->base = ao2_bump(old_snapshot->base);
+	}
+
+	if (ast_test_flag(ast_channel_snapshot_segment_flags(chan), AST_CHANNEL_SNAPSHOT_INVALIDATE_PEER)) {
+		/* The peer information has changed so update our snapshot */
+		snapshot->peer = channel_snapshot_peer_create(chan);
+		if (!snapshot->peer) {
+			ao2_ref(snapshot, -1);
+			return NULL;
+		}
+	} else {
+		snapshot->peer = ao2_bump(old_snapshot->peer);
+	}
+
+	/* Unfortunately both caller and connected information do not have an enforced contract with
+	 * the channel API. This has allowed consumers to directly get the caller or connected structure
+	 * and manipulate it. Until such time as there is an enforced contract (which is being tracked under
+	 * ASTERISK-28164) they are each regenerated every time a channel snapshot is created.
+	 */
+	snapshot->caller = channel_snapshot_caller_create(chan);
+	if (!snapshot->caller) {
+		ao2_ref(snapshot, -1);
+		return NULL;
+	}
+
+	snapshot->connected = channel_snapshot_connected_create(chan);
+	if (!snapshot->connected) {
+		ao2_ref(snapshot, -1);
+		return NULL;
+	}
+
+	if (ast_test_flag(ast_channel_snapshot_segment_flags(chan), AST_CHANNEL_SNAPSHOT_INVALIDATE_BRIDGE)) {
+		/* The bridge has changed so update our snapshot */
+		snapshot->bridge = channel_snapshot_bridge_create(chan);
+		if (!snapshot->bridge) {
+			ao2_ref(snapshot, -1);
+			return NULL;
+		}
+	} else {
+		snapshot->bridge = ao2_bump(old_snapshot->bridge);
+	}
+
+	if (ast_test_flag(ast_channel_snapshot_segment_flags(chan), AST_CHANNEL_SNAPSHOT_INVALIDATE_DIALPLAN)) {
+		/* The dialplan information has changed so update our snapshot */
+		snapshot->dialplan = channel_snapshot_dialplan_create(chan);
+		if (!snapshot->dialplan) {
+			ao2_ref(snapshot, -1);
+			return NULL;
+		}
+	} else {
+		snapshot->dialplan = ao2_bump(old_snapshot->dialplan);
+	}
+
+	if (ast_test_flag(ast_channel_snapshot_segment_flags(chan), AST_CHANNEL_SNAPSHOT_INVALIDATE_HANGUP)) {
+		/* The hangup information has changed so update our snapshot */
+		snapshot->hangup = channel_snapshot_hangup_create(chan);
+		if (!snapshot->hangup) {
+			ao2_ref(snapshot, -1);
+			return NULL;
+		}
+	} else {
+		snapshot->hangup = ao2_bump(old_snapshot->hangup);
+	}
+
+	snapshot->state = ast_channel_state(chan);
+	snapshot->amaflags = ast_channel_amaflags(chan);
+	ast_copy_flags(&snapshot->flags, ast_channel_flags(chan), 0xFFFFFFFF);
+	ast_set_flag(&snapshot->softhangup_flags, ast_channel_softhangup_internal_flag(chan));
+
+	/* These have to be recreated as they may have changed, unfortunately */
+	snapshot->manager_vars = ast_channel_get_manager_vars(chan);
+	snapshot->ari_vars = ast_channel_get_ari_vars(chan);
+
+	return snapshot;
+}
+
+static void channel_snapshot_update_dtor(void *obj)
+{
+	struct ast_channel_snapshot_update *update = obj;
+
+	ao2_cleanup(update->old_snapshot);
+	ao2_cleanup(update->new_snapshot);
+}
+
+static struct ast_channel_snapshot_update *channel_snapshot_update_create(struct ast_channel *chan)
+{
+	struct ast_channel_snapshot_update *update;
+
+	update = ao2_alloc_options(sizeof(*update), channel_snapshot_update_dtor,
+		AO2_ALLOC_OPT_LOCK_NOLOCK);
+	if (!update) {
+		return NULL;
+	}
+
+	update->old_snapshot = ao2_bump(ast_channel_snapshot(chan));
+	update->new_snapshot = ast_channel_snapshot_create(chan);
+	if (!update->new_snapshot) {
+		ao2_ref(update, -1);
+		return NULL;
+	}
+
+	return update;
+}
+
 static void publish_message_for_channel_topics(struct stasis_message *message, struct ast_channel *chan)
 {
 	if (chan) {
@@ -521,7 +795,7 @@ struct stasis_message *ast_channel_blob_create(struct ast_channel *chan,
 		return NULL;
 	}
 
-	snapshot = chan ? ast_channel_snapshot_create(chan) : NULL;
+	snapshot = chan ? ao2_bump(ast_channel_snapshot(chan)) : NULL;
 	msg = create_channel_blob_message(snapshot, type, blob);
 	ao2_cleanup(snapshot);
 	return msg;
@@ -628,38 +902,48 @@ struct ast_multi_channel_blob *ast_multi_channel_blob_create(struct ast_json *bl
 
 struct ast_channel_snapshot *ast_channel_snapshot_get_latest(const char *uniqueid)
 {
-	struct stasis_message *message;
-	struct ast_channel_snapshot *snapshot;
-
 	ast_assert(!ast_strlen_zero(uniqueid));
 
-	message = stasis_cache_get(ast_channel_cache(), ast_channel_snapshot_type(),
-		uniqueid);
-	if (!message) {
-		return NULL;
-	}
-
-	snapshot = ao2_bump(stasis_message_data(message));
-	ao2_ref(message, -1);
-	return snapshot;
+	return ao2_find(channel_cache, uniqueid, OBJ_SEARCH_KEY);
 }
 
 struct ast_channel_snapshot *ast_channel_snapshot_get_latest_by_name(const char *name)
 {
-	struct stasis_message *message;
-	struct ast_channel_snapshot *snapshot;
-
 	ast_assert(!ast_strlen_zero(name));
 
-	message = stasis_cache_get(ast_channel_cache_by_name(), ast_channel_snapshot_type(),
-		name);
-	if (!message) {
-		return NULL;
+	return ao2_find(channel_cache_by_name, name, OBJ_SEARCH_KEY);
+}
+
+void ast_channel_publish_final_snapshot(struct ast_channel *chan)
+{
+	struct ast_channel_snapshot_update *update;
+	struct stasis_message *message;
+
+	if (!ast_channel_snapshot_type()) {
+		return;
 	}
 
-	snapshot = ao2_bump(stasis_message_data(message));
+	update = channel_snapshot_update_create(chan);
+	if (!update) {
+		return;
+	}
+
+	message = stasis_message_create(ast_channel_snapshot_type(), update);
+	/* In the success path message holds a reference to update so it will be valid
+	 * for the lifetime of this function until the end.
+	 */
+	ao2_ref(update, -1);
+	if (!message) {
+		return;
+	}
+
+	ao2_unlink(channel_cache, update->old_snapshot);
+	ao2_unlink(channel_cache_by_name, update->old_snapshot);
+
+	ast_channel_snapshot_set(chan, NULL);
+
+	stasis_publish(ast_channel_topic(chan), message);
 	ao2_ref(message, -1);
-	return snapshot;
 }
 
 static void channel_role_snapshot_dtor(void *obj)
@@ -762,9 +1046,15 @@ void ast_channel_stage_snapshot_done(struct ast_channel *chan)
 	ast_channel_publish_snapshot(chan);
 }
 
+void ast_channel_snapshot_invalidate_segment(struct ast_channel *chan,
+	enum ast_channel_snapshot_segment_invalidation segment)
+{
+	ast_set_flag(ast_channel_snapshot_segment_flags(chan), segment);
+}
+
 void ast_channel_publish_snapshot(struct ast_channel *chan)
 {
-	struct ast_channel_snapshot *snapshot;
+	struct ast_channel_snapshot_update *update;
 	struct stasis_message *message;
 
 	if (!ast_channel_snapshot_type()) {
@@ -775,17 +1065,53 @@ void ast_channel_publish_snapshot(struct ast_channel *chan)
 		return;
 	}
 
-	snapshot = ast_channel_snapshot_create(chan);
-	if (!snapshot) {
+	update = channel_snapshot_update_create(chan);
+	if (!update) {
 		return;
 	}
 
-	message = stasis_message_create(ast_channel_snapshot_type(), snapshot);
-	ao2_ref(snapshot, -1);
+	/* If an old snapshot exists and is the same as this newly created one don't bother
+	 * raising a message as it hasn't changed.
+	 */
+	if (update->old_snapshot && !memcmp(update->old_snapshot, update->new_snapshot, sizeof(struct ast_channel_snapshot))) {
+		ao2_ref(update, -1);
+		return;
+	}
+
+	message = stasis_message_create(ast_channel_snapshot_type(), update);
+	/* In the success path message holds a reference to update so it will be valid
+	 * for the lifetime of this function until the end.
+	 */
+	ao2_ref(update, -1);
 	if (!message) {
 		return;
 	}
 
+	/* We lock these ourselves so that the update is atomic and there isn't time where a
+	 * snapshot is not in the cache.
+	 */
+	ao2_wrlock(channel_cache);
+	if (update->old_snapshot) {
+		ao2_unlink_flags(channel_cache, update->old_snapshot, OBJ_NOLOCK);
+	}
+	ao2_link_flags(channel_cache, update->new_snapshot, OBJ_NOLOCK);
+	ao2_unlock(channel_cache);
+
+	/* The same applies here. */
+	ao2_wrlock(channel_cache_by_name);
+	if (update->old_snapshot) {
+		ao2_unlink_flags(channel_cache_by_name, update->old_snapshot, OBJ_NOLOCK);
+	}
+	ao2_link_flags(channel_cache_by_name, update->new_snapshot, OBJ_NOLOCK);
+	ao2_unlock(channel_cache_by_name);
+
+	ast_channel_snapshot_set(chan, update->new_snapshot);
+
+	/* As this is now the new snapshot any existing invalidated segments have been
+	 * created fresh and are up to date.
+	 */
+	ast_clear_flag(ast_channel_snapshot_segment_flags(chan), AST_FLAGS_ALL);
+
 	ast_assert(ast_channel_topic(chan) != NULL);
 	stasis_publish(ast_channel_topic(chan), message);
 	ao2_ref(message, -1);
@@ -841,13 +1167,8 @@ void ast_channel_publish_varset(struct ast_channel *chan, const char *name, cons
 		ast_channel_publish_snapshot(chan);
 	}
 
-	if (chan) {
-		ast_channel_publish_cached_blob(chan, ast_channel_varset_type(), blob);
-	} else {
-		/* This function is NULL safe for global variables */
-		ast_channel_publish_blob(NULL, ast_channel_varset_type(), blob);
-	}
-
+	/* This function is NULL safe for global variables */
+	ast_channel_publish_blob(chan, ast_channel_varset_type(), blob);
 	ast_json_unref(blob);
 }
 
@@ -931,36 +1252,6 @@ static struct ast_manager_event_blob *agent_logoff_to_ami(struct stasis_message
 	return ev;
 }
 
-void ast_publish_channel_state(struct ast_channel *chan)
-{
-	struct ast_channel_snapshot *snapshot;
-	struct stasis_message *message;
-
-	if (!ast_channel_snapshot_type()) {
-		return;
-	}
-
-	ast_assert(chan != NULL);
-	if (!chan) {
-		return;
-	}
-
-	snapshot = ast_channel_snapshot_create(chan);
-	if (!snapshot) {
-		return;
-	}
-
-	message = stasis_message_create(ast_channel_snapshot_type(), snapshot);
-	ao2_ref(snapshot, -1);
-	if (!message) {
-		return;
-	}
-
-	ast_assert(ast_channel_topic(chan) != NULL);
-	stasis_publish(ast_channel_topic(chan), message);
-	ao2_ref(message, -1);
-}
-
 struct ast_json *ast_channel_snapshot_to_json(
 	const struct ast_channel_snapshot *snapshot,
 	const struct stasis_message_sanitizer *sanitize)
@@ -980,21 +1271,21 @@ struct ast_json *ast_channel_snapshot_to_json(
 		"  s: o, s: o, s: s,"
 		"  s: o, s: o, s: s }",
 		/* First line */
-		"id", snapshot->uniqueid,
-		"name", snapshot->name,
+		"id", snapshot->base->uniqueid,
+		"name", snapshot->base->name,
 		"state", ast_state2str(snapshot->state),
 		/* Second line */
 		"caller", ast_json_name_number(
-			snapshot->caller_name, snapshot->caller_number),
+			snapshot->caller->name, snapshot->caller->number),
 		"connected", ast_json_name_number(
-			snapshot->connected_name, snapshot->connected_number),
-		"accountcode", snapshot->accountcode,
+			snapshot->connected->name, snapshot->connected->number),
+		"accountcode", snapshot->base->accountcode,
 		/* Third line */
 		"dialplan", ast_json_dialplan_cep_app(
-			snapshot->context, snapshot->exten, snapshot->priority,
-			snapshot->appl, snapshot->data),
-		"creationtime", ast_json_timeval(snapshot->creationtime, NULL),
-		"language", snapshot->language);
+			snapshot->dialplan->context, snapshot->dialplan->exten, snapshot->dialplan->priority,
+			snapshot->dialplan->appl, snapshot->dialplan->data),
+		"creationtime", ast_json_timeval(snapshot->base->creationtime, NULL),
+		"language", snapshot->base->language);
 
 	if (snapshot->ari_vars && !AST_LIST_EMPTY(snapshot->ari_vars)) {
 		ast_json_object_set(json_chan, "channelvars", ast_json_channel_vars(snapshot->ari_vars));
@@ -1014,14 +1305,14 @@ int ast_channel_snapshot_cep_equal(
 	 * application is set. Since empty application is invalid, we treat
 	 * setting the application from nothing as a CEP change.
 	 */
-	if (ast_strlen_zero(old_snapshot->appl) &&
-	    !ast_strlen_zero(new_snapshot->appl)) {
+	if (ast_strlen_zero(old_snapshot->dialplan->appl) &&
+	    !ast_strlen_zero(new_snapshot->dialplan->appl)) {
 		return 0;
 	}
 
-	return old_snapshot->priority == new_snapshot->priority &&
-		strcmp(old_snapshot->context, new_snapshot->context) == 0 &&
-		strcmp(old_snapshot->exten, new_snapshot->exten) == 0;
+	return old_snapshot->dialplan->priority == new_snapshot->dialplan->priority &&
+		strcmp(old_snapshot->dialplan->context, new_snapshot->dialplan->context) == 0 &&
+		strcmp(old_snapshot->dialplan->exten, new_snapshot->dialplan->exten) == 0;
 }
 
 int ast_channel_snapshot_caller_id_equal(
@@ -1030,8 +1321,8 @@ int ast_channel_snapshot_caller_id_equal(
 {
 	ast_assert(old_snapshot != NULL);
 	ast_assert(new_snapshot != NULL);
-	return strcmp(old_snapshot->caller_number, new_snapshot->caller_number) == 0 &&
-		strcmp(old_snapshot->caller_name, new_snapshot->caller_name) == 0;
+	return strcmp(old_snapshot->caller->number, new_snapshot->caller->number) == 0 &&
+		strcmp(old_snapshot->caller->name, new_snapshot->caller->name) == 0;
 }
 
 int ast_channel_snapshot_connected_line_equal(
@@ -1040,8 +1331,8 @@ int ast_channel_snapshot_connected_line_equal(
 {
 	ast_assert(old_snapshot != NULL);
 	ast_assert(new_snapshot != NULL);
-	return strcmp(old_snapshot->connected_number, new_snapshot->connected_number) == 0 &&
-		strcmp(old_snapshot->connected_name, new_snapshot->connected_name) == 0;
+	return strcmp(old_snapshot->connected->number, new_snapshot->connected->number) == 0 &&
+		strcmp(old_snapshot->connected->name, new_snapshot->connected->name) == 0;
 }
 
 static struct ast_json *channel_blob_to_json(
@@ -1307,7 +1598,6 @@ STASIS_MESSAGE_TYPE_DEFN(ast_channel_hold_type,
 STASIS_MESSAGE_TYPE_DEFN(ast_channel_unhold_type,
 	.to_json = unhold_to_json,
 	);
-STASIS_MESSAGE_TYPE_DEFN(ast_channel_flash_type);
 STASIS_MESSAGE_TYPE_DEFN(ast_channel_chanspy_start_type);
 STASIS_MESSAGE_TYPE_DEFN(ast_channel_chanspy_stop_type);
 STASIS_MESSAGE_TYPE_DEFN(ast_channel_fax_type);
@@ -1338,12 +1628,12 @@ STASIS_MESSAGE_TYPE_DEFN(ast_channel_talking_stop,
 
 static void stasis_channels_cleanup(void)
 {
-	stasis_caching_unsubscribe_and_join(channel_by_name_topic);
-	channel_by_name_topic = NULL;
+	ao2_cleanup(channel_topic_all);
+	channel_topic_all = NULL;
+	ao2_cleanup(channel_cache);
+	channel_cache = NULL;
 	ao2_cleanup(channel_cache_by_name);
 	channel_cache_by_name = NULL;
-	ao2_cleanup(channel_cache_all);
-	channel_cache_all = NULL;
 
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_snapshot_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_dial_type);
@@ -1352,7 +1642,6 @@ static void stasis_channels_cleanup(void)
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_masquerade_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_dtmf_begin_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_dtmf_end_type);
-	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_flash_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_hold_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_unhold_type);
 	STASIS_MESSAGE_TYPE_CLEANUP(ast_channel_chanspy_start_type);
@@ -1378,36 +1667,34 @@ int ast_stasis_channels_init(void)
 
 	ast_register_cleanup(stasis_channels_cleanup);
 
-	channel_cache_all = stasis_cp_all_create("channel:all",
-		channel_snapshot_get_id);
-	if (!channel_cache_all) {
+	channel_topic_all = stasis_topic_create("channel:all");
+	if (!channel_topic_all) {
 		return -1;
 	}
-	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_agent_login_type);
-	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_agent_logoff_type);
 
-	channel_cache_by_name = stasis_cache_create(channel_snapshot_get_name);
+	channel_cache = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK,
+		0, AST_NUM_CHANNEL_BUCKETS, channel_snapshot_uniqueid_hash_cb,
+		NULL, channel_snapshot_uniqueid_cmp_cb);
+	if (!channel_cache) {
+		return -1;
+	}
+
+	channel_cache_by_name = ao2_container_alloc_hash(AO2_ALLOC_OPT_LOCK_RWLOCK,
+		0, AST_NUM_CHANNEL_BUCKETS, channel_snapshot_hash_cb,
+		NULL, channel_snapshot_cmp_cb);
 	if (!channel_cache_by_name) {
 		return -1;
 	}
 
-	/* This should be initialized before the caching topic */
+	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_agent_login_type);
+	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_agent_logoff_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_snapshot_type);
-
-	channel_by_name_topic = stasis_caching_topic_create(
-		stasis_cp_all_topic(channel_cache_all),
-		channel_cache_by_name);
-	if (!channel_by_name_topic) {
-		return -1;
-	}
-
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_dial_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_varset_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_hangup_request_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_masquerade_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_dtmf_begin_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_dtmf_end_type);
-	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_flash_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_hold_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_unhold_type);
 	res |= STASIS_MESSAGE_TYPE_INIT(ast_channel_chanspy_start_type);
