@@ -203,6 +203,176 @@ struct ast_sip_endpoint_formatter endpoint_transport_formatter = {
 	.format_ami = format_ami_endpoint_transport
 };
 
+int ast_sip_transport_state_set_transport(const char *transport_name, pjsip_transport *transport)
+{
+	struct ast_sip_transport_state *transport_state;
+
+	/* To make it easier on callers we allow an empty transport name */
+	if (ast_strlen_zero(transport_name)) {
+		return 0;
+	}
+
+	transport_state = ast_sip_get_transport_state(transport_name);
+	if (!transport_state) {
+		return -1;
+	}
+
+	if (!transport_state->flow) {
+		ao2_ref(transport_state, -1);
+		return 0;
+	}
+
+	ao2_lock(transport_state);
+	if (transport_state->transport != transport) {
+		if (transport_state->transport) {
+			pjsip_transport_dec_ref(transport_state->transport);
+		}
+		transport_state->transport = transport;
+		if (transport_state->transport) {
+			pjsip_transport_add_ref(transport_state->transport);
+		}
+	}
+	ao2_unlock(transport_state);
+
+	ao2_ref(transport_state, -1);
+
+	return 0;
+}
+
+int ast_sip_transport_state_set_preferred_identity(const char *transport_name, const char *identity)
+{
+	struct ast_sip_transport_state *transport_state;
+
+	if (ast_strlen_zero(transport_name)) {
+		return 0;
+	}
+
+	transport_state = ast_sip_get_transport_state(transport_name);
+	if (!transport_state) {
+		return -1;
+	}
+
+	if (!transport_state->flow) {
+		ao2_ref(transport_state, -1);
+		return 0;
+	}
+
+	ao2_lock(transport_state);
+	ast_free(transport_state->preferred_identity);
+	transport_state->preferred_identity = ast_strdup(identity);
+	ao2_unlock(transport_state);
+
+	ao2_ref(transport_state, -1);
+
+	return 0;
+}
+
+int ast_sip_transport_state_set_service_routes(const char *transport_name, struct ast_sip_service_route_vector *service_routes)
+{
+	struct ast_sip_transport_state *transport_state;
+
+	if (ast_strlen_zero(transport_name)) {
+		ast_sip_service_route_vector_destroy(service_routes);
+		return 0;
+	}
+
+	transport_state = ast_sip_get_transport_state(transport_name);
+	if (!transport_state) {
+		ast_sip_service_route_vector_destroy(service_routes);
+		return -1;
+	}
+
+	if (!transport_state->flow) {
+		ao2_ref(transport_state, -1);
+		ast_sip_service_route_vector_destroy(service_routes);
+		return 0;
+	}
+
+	ao2_lock(transport_state);
+	ast_sip_service_route_vector_destroy(transport_state->service_routes);
+	transport_state->service_routes = service_routes;
+	ao2_unlock(transport_state);
+
+	ao2_ref(transport_state, -1);
+
+	return 0;
+}
+
+void ast_sip_message_apply_transport(const char *transport_name, pjsip_tx_data *tdata)
+{
+	struct ast_sip_transport_state *transport_state;
+
+	if (ast_strlen_zero(transport_name)) {
+		return;
+	}
+
+	/* We only currently care about requests that are of the INVITE, CANCEL, or OPTIONS
+	 * type but in the future we could support other messages.
+	 */
+	if (tdata->msg->type != PJSIP_REQUEST_MSG ||
+		(pjsip_method_cmp(&tdata->msg->line.req.method, &pjsip_invite_method) &&
+		pjsip_method_cmp(&tdata->msg->line.req.method, &pjsip_cancel_method) &&
+		pjsip_method_cmp(&tdata->msg->line.req.method, &pjsip_options_method))) {
+		return;
+	}
+
+	transport_state = ast_sip_get_transport_state(transport_name);
+	if (!transport_state) {
+		return;
+	}
+
+	if (!transport_state->flow) {
+		ao2_ref(transport_state, -1);
+		return;
+	}
+
+	ao2_lock(transport_state);
+
+	/* If a Preferred Identity has been set then add it to the request */
+	if (transport_state->preferred_identity) {
+		ast_sip_add_header(tdata, "P-Preferred-Identity", transport_state->preferred_identity);
+	}
+
+	/* If Service Routes have been set then add them to the request */
+	if (transport_state->service_routes) {
+		int idx;
+
+		for (idx = 0; idx < AST_VECTOR_SIZE(transport_state->service_routes); ++idx) {
+			char *service_route = AST_VECTOR_GET(transport_state->service_routes, idx);
+
+			ast_sip_add_header(tdata, "Route", service_route);
+		}
+	}
+
+	ao2_unlock(transport_state);
+
+	ao2_ref(transport_state, -1);
+}
+
+struct ast_sip_service_route_vector *ast_sip_service_route_vector_alloc(void)
+{
+	struct ast_sip_service_route_vector *service_routes;
+
+	service_routes = ast_calloc(1, sizeof(*service_routes));
+	if (!service_routes) {
+		return NULL;
+	}
+
+	AST_VECTOR_INIT(service_routes, 0);
+
+	return service_routes;
+}
+
+void ast_sip_service_route_vector_destroy(struct ast_sip_service_route_vector *service_routes)
+{
+	if (!service_routes) {
+		return;
+	}
+
+	AST_VECTOR_CALLBACK_VOID(service_routes, ast_free);
+	ast_free(service_routes);
+}
+
 static void set_qos(struct ast_sip_transport *transport, pj_qos_params *qos)
 {
 	int tos_as_dscp = transport->tos >> 2;
@@ -407,22 +577,6 @@ static void copy_state_to_transport(struct ast_sip_transport *transport)
 	memcpy(&transport->external_address, &transport->state->external_signaling_address, sizeof(transport->external_signaling_address));
 }
 
-#ifdef HAVE_PJSIP_TLS_TRANSPORT_RESTART
-static int file_stat_cmp(const struct stat *old_stat, const struct stat *new_stat)
-{
-	return old_stat->st_size != new_stat->st_size
-		|| old_stat->st_mtime != new_stat->st_mtime
-#if defined(HAVE_STRUCT_STAT_ST_MTIM)
-		|| old_stat->st_mtim.tv_nsec != new_stat->st_mtim.tv_nsec
-#elif defined(HAVE_STRUCT_STAT_ST_MTIMENSEC)
-		|| old_stat->st_mtimensec != new_stat->st_mtimensec
-#elif defined(HAVE_STRUCT_STAT_ST_MTIMESPEC)
-		|| old_stat->st_mtimespec.tv_nsec != new_stat->st_mtimespec.tv_nsec
-#endif
-        ;
-}
-#endif
-
 static int has_state_changed(struct ast_sip_transport_state *a, struct ast_sip_transport_state *b)
 {
 	if (a->type != b->type) {
@@ -464,12 +618,6 @@ static int has_state_changed(struct ast_sip_transport_state *a, struct ast_sip_t
 		return -1;
 	}
 
-#ifdef HAVE_PJSIP_TLS_TRANSPORT_RESTART
-	if (file_stat_cmp(&a->cert_file_stat, &b->cert_file_stat) || file_stat_cmp(&a->privkey_file_stat, &b->privkey_file_stat)) {
-		return -1;
-	}
-#endif
-
 	return 0;
 }
 
@@ -511,12 +659,6 @@ static int transport_apply(const struct ast_sorcery *sorcery, void *obj)
 		return -1;
 	}
 
-	if (transport->async_operations != 1) {
-		ast_log(LOG_WARNING, "The async_operations setting on transport '%s' has been set to '%d'. The setting can no longer be set and is always 1.\n",
-			transport_id, transport->async_operations);
-		transport->async_operations = 1;
-	}
-
 	perm_state = find_internal_state_by_transport(transport);
 	if (perm_state) {
 		ast_sorcery_diff(sorcery, perm_state->transport, transport, &changes);
@@ -527,6 +669,7 @@ static int transport_apply(const struct ast_sorcery *sorcery, void *obj)
 			ao2_replace(perm_state->transport, transport);
 			return 0;
 		}
+
 		/* If we aren't allowed to reload then we copy values that can't be changed from perm_state */
 		if (!transport->allow_reload) {
 			memcpy(&temp_state->state->host, &perm_state->state->host, sizeof(temp_state->state->host));
@@ -535,7 +678,7 @@ static int transport_apply(const struct ast_sorcery *sorcery, void *obj)
 		}
 	}
 
-	if (!perm_state || transport->allow_reload) {
+	if (!transport->flow && (!perm_state || transport->allow_reload)) {
 		if (temp_state->state->host.addr.sa_family != PJ_AF_INET && temp_state->state->host.addr.sa_family != PJ_AF_INET6) {
 			ast_log(LOG_ERROR, "Transport '%s' could not be started as binding not specified\n", transport_id);
 			return -1;
@@ -582,35 +725,23 @@ static int transport_apply(const struct ast_sorcery *sorcery, void *obj)
 		}
 	}
 
-	if (!transport->allow_reload && perm_state) {
+	if (transport->flow) {
+		pj_str_t address;
+
+		ast_debug(1, "Ignoring any bind configuration on transport '%s' as it is a child of another\n",
+			transport_id);
+		pj_sockaddr_parse(pj_AF_UNSPEC(), 0, pj_cstr(&address, "0.0.0.0"), &temp_state->state->host);
+
+		temp_state->state->flow = 1;
+		res = PJ_SUCCESS;
+	} else if (!transport->allow_reload && perm_state) {
 		/* We inherit the transport from perm state, untouched */
-#ifdef HAVE_PJSIP_TLS_TRANSPORT_RESTART
-		ast_log(LOG_NOTICE, "Transport '%s' is not fully reloadable, not reloading: protocol, bind, TLS (everything but certificate and private key if filename is unchanged), TCP, ToS, or CoS options.\n", transport_id);
-		/* If this is a TLS transport and the certificate or private key has changed, then restart the transport so it uses the new one */
-		if (transport->type == AST_TRANSPORT_TLS) {
-			if (strcmp(perm_state->transport->cert_file, temp_state->transport->cert_file) ||
-				strcmp(perm_state->transport->privkey_file, temp_state->transport->privkey_file)) {
-				ast_log(LOG_ERROR, "Unable to restart TLS transport '%s' as certificate or private key filename has changed\n",
-					transport_id);
-			} else if (file_stat_cmp(&perm_state->state->cert_file_stat, &temp_state->state->cert_file_stat) ||
-				file_stat_cmp(&perm_state->state->privkey_file_stat, &temp_state->state->privkey_file_stat)) {
-				if (pjsip_tls_transport_restart(perm_state->state->factory, &perm_state->state->host, NULL) != PJ_SUCCESS) {
-					ast_log(LOG_ERROR, "Failed to restart TLS transport '%s'\n", transport_id);
-				} else {
-					sprintf(perm_state->state->factory->info, "%s", transport_id);
-				}
-			}
-		}
-#else
-		ast_log(LOG_NOTICE, "Transport '%s' is not fully reloadable, not reloading: protocol, bind, TLS, TCP, ToS, or CoS options.\n", transport_id);
-#endif
+		ast_log(LOG_WARNING, "Transport '%s' is not fully reloadable, not reloading: protocol, bind, TLS, TCP, ToS, or CoS options.\n", transport_id);
 		temp_state->state->transport = perm_state->state->transport;
 		perm_state->state->transport = NULL;
-		temp_state->state->factory = perm_state->state->factory;
-		perm_state->state->factory = NULL;
-
 		res = PJ_SUCCESS;
 	} else if (transport->type == AST_TRANSPORT_UDP) {
+
 		for (i = 0; i < BIND_TRIES && res != PJ_SUCCESS; i++) {
 			if (perm_state && perm_state->state && perm_state->state->transport) {
 				pjsip_udp_transport_pause(perm_state->state->transport,
@@ -700,20 +831,6 @@ static int transport_apply(const struct ast_sorcery *sorcery, void *obj)
 				&temp_state->state->host, NULL, transport->async_operations,
 				&temp_state->state->factory);
 		}
-
-		if (res == PJ_SUCCESS) {
-			/*
-			 * PJSIP uses 100 bytes to store information, and during a restart will repopulate
-			 * the field so ensure there is sufficient space - even though we'll revert it after.
-			 */
-			temp_state->state->factory->info = pj_pool_alloc(
-				temp_state->state->factory->pool, (MAX(MAX_OBJECT_FIELD, 100) + 1));
-			/*
-			 * Store transport id on the factory instance so it can be used
-			 * later to look up the transport state.
-			 */
-			sprintf(temp_state->state->factory->info, "%s", transport_id);
-		}
 #else
 		ast_log(LOG_ERROR, "Transport: %s: PJSIP has not been compiled with TLS transport support, ensure OpenSSL development packages are installed\n",
 			ast_sorcery_object_get_id(obj));
@@ -783,34 +900,18 @@ static int transport_tls_file_handler(const struct aco_option *opt, struct ast_v
 		ast_string_field_set(transport, ca_list_file, var->value);
 	} else if (!strcasecmp(var->name, "ca_list_path")) {
 #ifdef HAVE_PJ_SSL_CERT_LOAD_FROM_FILES2
-		state->tls.ca_list_path = pj_str((char *)var->value);
+		state->tls.ca_list_path = pj_str((char*)var->value);
 		ast_string_field_set(transport, ca_list_path, var->value);
 #else
 		ast_log(LOG_WARNING, "Asterisk has been built against a version of pjproject that does not "
 				"support the 'ca_list_path' option. Please upgrade to version 2.4 or later.\n");
 #endif
 	} else if (!strcasecmp(var->name, "cert_file")) {
-		state->tls.cert_file = pj_str((char *)var->value);
+		state->tls.cert_file = pj_str((char*)var->value);
 		ast_string_field_set(transport, cert_file, var->value);
-#ifdef HAVE_PJSIP_TLS_TRANSPORT_RESTART
-		if (stat(var->value, &state->cert_file_stat)) {
-			ast_log(LOG_ERROR, "Failed to stat certificate file '%s' for transport '%s' due to '%s'\n",
-				var->value, ast_sorcery_object_get_id(obj), strerror(errno));
-			return -1;
-		}
-		ast_sorcery_object_set_has_dynamic_contents(transport);
-#endif
 	} else if (!strcasecmp(var->name, "priv_key_file")) {
-		state->tls.privkey_file = pj_str((char *)var->value);
+		state->tls.privkey_file = pj_str((char*)var->value);
 		ast_string_field_set(transport, privkey_file, var->value);
-#ifdef HAVE_PJSIP_TLS_TRANSPORT_RESTART
-		if (stat(var->value, &state->privkey_file_stat)) {
-			ast_log(LOG_ERROR, "Failed to stat private key file '%s' for transport '%s' due to '%s'\n",
-				var->value, ast_sorcery_object_get_id(obj), strerror(errno));
-			return -1;
-		}
-		ast_sorcery_object_set_has_dynamic_contents(transport);
-#endif
 	}
 
 	return 0;
@@ -862,18 +963,23 @@ static int transport_protocol_handler(const struct aco_option *opt, struct ast_v
 		return -1;
 	}
 
-	if (!strcasecmp(var->value, "udp")) {
-		transport->type = AST_TRANSPORT_UDP;
-	} else if (!strcasecmp(var->value, "tcp")) {
-		transport->type = AST_TRANSPORT_TCP;
-	} else if (!strcasecmp(var->value, "tls")) {
-		transport->type = AST_TRANSPORT_TLS;
-	} else if (!strcasecmp(var->value, "ws")) {
-		transport->type = AST_TRANSPORT_WS;
-	} else if (!strcasecmp(var->value, "wss")) {
-		transport->type = AST_TRANSPORT_WSS;
+	if (!strcasecmp(var->value, "flow")) {
+		transport->flow = 1;
 	} else {
-		return -1;
+		if (!strcasecmp(var->value, "udp")) {
+			transport->type = AST_TRANSPORT_UDP;
+		} else if (!strcasecmp(var->value, "tcp")) {
+			transport->type = AST_TRANSPORT_TCP;
+		} else if (!strcasecmp(var->value, "tls")) {
+			transport->type = AST_TRANSPORT_TLS;
+		} else if (!strcasecmp(var->value, "ws")) {
+			transport->type = AST_TRANSPORT_WS;
+		} else if (!strcasecmp(var->value, "wss")) {
+			transport->type = AST_TRANSPORT_WSS;
+		} else {
+			return -1;
+		}
+		transport->flow = 0;
 	}
 
 	state->type = transport->type;
@@ -893,7 +999,9 @@ static int transport_protocol_to_str(const void *obj, const intptr_t *args, char
 {
 	const struct ast_sip_transport *transport = obj;
 
-	if (ARRAY_IN_BOUNDS(transport->type, transport_types)) {
+	if (transport->flow) {
+		*buf = ast_strdup("flow");
+	} else if (ARRAY_IN_BOUNDS(transport->type, transport_types)) {
 		*buf = ast_strdup(transport_types[transport->type]);
 	}
 
@@ -947,13 +1055,11 @@ static int transport_tls_bool_handler(const struct aco_option *opt, struct ast_v
 	}
 
 	if (!strcasecmp(var->name, "verify_server")) {
-		state->verify_server = ast_true(var->value);
+		state->tls.verify_server = ast_true(var->value) ? PJ_TRUE : PJ_FALSE;
 	} else if (!strcasecmp(var->name, "verify_client")) {
 		state->tls.verify_client = ast_true(var->value) ? PJ_TRUE : PJ_FALSE;
 	} else if (!strcasecmp(var->name, "require_client_cert")) {
 		state->tls.require_client_cert = ast_true(var->value) ? PJ_TRUE : PJ_FALSE;
-	} else if (!strcasecmp(var->name, "allow_wildcard_certs")) {
-		state->allow_wildcard_certs = ast_true(var->value);
 	} else {
 		return -1;
 	}
@@ -970,7 +1076,7 @@ static int verify_server_to_str(const void *obj, const intptr_t *args, char **bu
 		return -1;
 	}
 
-	*buf = ast_strdup(AST_YESNO(state->verify_server));
+	*buf = ast_strdup(AST_YESNO(state->tls.verify_server));
 
 	return 0;
 }
@@ -1003,20 +1109,6 @@ static int require_client_cert_to_str(const void *obj, const intptr_t *args, cha
 	return 0;
 }
 
-static int allow_wildcard_certs_to_str(const void *obj, const intptr_t *args, char **buf)
-{
-	struct ast_sip_transport_state *state = find_state_by_transport(obj);
-
-	if (!state) {
-		return -1;
-	}
-
-	*buf = ast_strdup(AST_YESNO(state->allow_wildcard_certs));
-	ao2_ref(state, -1);
-
-	return 0;
-}
-
 /*! \brief Custom handler for TLS method setting */
 static int transport_tls_method_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
 {
@@ -1080,8 +1172,8 @@ static int tls_method_to_str(const void *obj, const intptr_t *args, char **buf)
 	return 0;
 }
 
-#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
 /*! \brief Helper function which turns a cipher name into an identifier */
+#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
 static pj_ssl_cipher cipher_name_to_id(const char *name)
 {
 	pj_ssl_cipher ciphers[PJ_SSL_SOCK_MAX_CIPHERS];
@@ -1103,17 +1195,17 @@ static pj_ssl_cipher cipher_name_to_id(const char *name)
 }
 #endif
 
-#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
 /*!
  * \internal
  * \brief Add a new cipher to the transport's cipher list array.
  *
- * \param state Which transport to add the cipher to.
+ * \param transport Which transport to add the cipher to.
  * \param name Cipher identifier name.
  *
  * \retval 0 on success.
  * \retval -1 on error.
  */
+#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
 static int transport_cipher_add(struct ast_sip_transport_state *state, const char *name)
 {
 	pj_ssl_cipher cipher;
@@ -1146,8 +1238,8 @@ static int transport_cipher_add(struct ast_sip_transport_state *state, const cha
 }
 #endif
 
-#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
 /*! \brief Custom handler for TLS cipher setting */
+#if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
 static int transport_tls_cipher_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
 {
 	struct ast_sip_transport *transport = obj;
@@ -1484,6 +1576,16 @@ struct ast_sip_transport_state *ast_sip_get_transport_state(const char *transpor
 	trans_state = ao2_bump(state->state);
 	ao2_ref(state, -1);
 
+	/* If this is a child transport see if the transport is actually dead */
+	if (trans_state->flow) {
+		ao2_lock(trans_state);
+		if (trans_state->transport && trans_state->transport->is_shutdown == PJ_TRUE) {
+			pjsip_transport_dec_ref(trans_state->transport);
+			trans_state->transport = NULL;
+		}
+		ao2_unlock(trans_state);
+	}
+
 	return trans_state;
 }
 
@@ -1549,7 +1651,6 @@ int ast_sip_initialize_sorcery_transport(void)
 	ast_sorcery_object_field_register_custom(sorcery, "transport", "verify_server", "", transport_tls_bool_handler, verify_server_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sorcery, "transport", "verify_client", "", transport_tls_bool_handler, verify_client_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sorcery, "transport", "require_client_cert", "", transport_tls_bool_handler, require_client_cert_to_str, NULL, 0, 0);
-	ast_sorcery_object_field_register_custom(sorcery, "transport", "allow_wildcard_certs", "", transport_tls_bool_handler, allow_wildcard_certs_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sorcery, "transport", "method", "", transport_tls_method_handler, tls_method_to_str, NULL, 0, 0);
 #if defined(PJ_HAS_SSL_SOCK) && PJ_HAS_SSL_SOCK != 0
 	ast_sorcery_object_field_register_custom(sorcery, "transport", "cipher", "", transport_tls_cipher_handler, transport_tls_cipher_to_str, NULL, 0, 0);
