@@ -21,7 +21,6 @@
 #include <pjsip.h>
 #include <pjsip_ua.h>
 
-#include "asterisk/res_geolocation.h"
 #include "asterisk/res_pjsip.h"
 #include "include/res_pjsip_private.h"
 #include "asterisk/res_pjsip_cli.h"
@@ -182,16 +181,9 @@ static int prack_handler(const struct aco_option *opt, struct ast_variable *var,
 
 	if (ast_true(var->value)) {
 		endpoint->extensions.flags |= PJSIP_INV_SUPPORT_100REL;
-		endpoint->rel100 = AST_SIP_100REL_SUPPORTED;
-	} else if (!strcasecmp(var->value, "peer_supported")) {
-		endpoint->extensions.flags |= PJSIP_INV_SUPPORT_100REL;
-		endpoint->rel100 = AST_SIP_100REL_PEER_SUPPORTED;
 	} else if (!strcasecmp(var->value, "required")) {
 		endpoint->extensions.flags |= PJSIP_INV_REQUIRE_100REL;
-		endpoint->rel100 = AST_SIP_100REL_REQUIRED;
-	} else if (ast_false(var->value)) {
-		endpoint->rel100 = AST_SIP_100REL_UNSUPPORTED;
-	} else {
+	} else if (!ast_false(var->value)){
 		return -1;
 	}
 
@@ -202,12 +194,10 @@ static int prack_to_str(const void *obj, const intptr_t *args, char **buf)
 {
 	const struct ast_sip_endpoint *endpoint = obj;
 
-	if (endpoint->rel100 == AST_SIP_100REL_SUPPORTED) {
-		*buf = "yes";
-	} else if (endpoint->rel100 == AST_SIP_100REL_PEER_SUPPORTED) {
-		*buf = "peer_supported";
-	} else if (endpoint->rel100 == AST_SIP_100REL_REQUIRED) {
+	if (endpoint->extensions.flags & PJSIP_INV_REQUIRE_100REL) {
 		*buf = "required";
+	} else if (endpoint->extensions.flags & PJSIP_INV_SUPPORT_100REL) {
+		*buf = "yes";
 	} else {
 		*buf = "no";
 	}
@@ -230,7 +220,7 @@ static int timers_handler(const struct aco_option *opt, struct ast_variable *var
 	} else if (!strcasecmp(var->value, "required")) {
 		endpoint->extensions.flags |= PJSIP_INV_REQUIRE_TIMER;
 	} else if (!strcasecmp(var->value, "always") || !strcasecmp(var->value, "forced")) {
-		endpoint->extensions.flags |= (PJSIP_INV_SUPPORT_TIMER | PJSIP_INV_ALWAYS_USE_TIMER);
+		endpoint->extensions.flags |= PJSIP_INV_ALWAYS_USE_TIMER;
 	} else if (!ast_false(var->value)) {
 		return -1;
 	}
@@ -256,31 +246,6 @@ static int timers_to_str(const void *obj, const intptr_t *args, char **buf)
 	return 0;
 }
 
-static int security_mechanism_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
-{
-	struct ast_sip_endpoint *endpoint = obj;
-
-	return ast_sip_security_mechanism_vector_init(&endpoint->security_mechanisms, var->value);
-}
-
-int ast_sip_set_security_negotiation(enum ast_sip_security_negotiation *security_negotiation, const char *val) {
-	if (!strcasecmp("no", val)) {
-		*security_negotiation = AST_SIP_SECURITY_NEG_NONE;
-	} else if (!strcasecmp("mediasec", val)) {
-		*security_negotiation = AST_SIP_SECURITY_NEG_MEDIASEC;
-	} else {
-		return -1;
-	}
-	return 0;
-}
-
-static int security_negotiation_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
-{
-	struct ast_sip_endpoint *endpoint = obj;
-
-	return ast_sip_set_security_negotiation(&endpoint->security_negotiation, var->value);
-}
-
 void ast_sip_auth_vector_destroy(struct ast_sip_auth_vector *auths)
 {
 	int i;
@@ -462,7 +427,7 @@ static int ident_handler(const struct aco_option *opt, struct ast_variable *var,
 			return -1;
 		}
 		if (endpoint->ident_method & method) {
-			/* We are already identifying by this method.  No need to do it again. */
+			/* We are already indentifying by this method.  No need to do it again. */
 			continue;
 		}
 
@@ -752,44 +717,6 @@ static int media_encryption_to_str(const void *obj, const intptr_t *args, char *
 	return 0;
 }
 
-static int stir_shaken_handler(const struct aco_option *opt, struct ast_variable *var, void *obj)
-{
-	struct ast_sip_endpoint *endpoint = obj;
-
-	if (!strcasecmp("off", var->value)) {
-		endpoint->stir_shaken = AST_SIP_STIR_SHAKEN_OFF;
-	} else if (!strcasecmp("attest", var->value)) {
-		endpoint->stir_shaken = AST_SIP_STIR_SHAKEN_ATTEST;
-	} else if (!strcasecmp("verify", var->value)) {
-		endpoint->stir_shaken = AST_SIP_STIR_SHAKEN_VERIFY;
-	} else if (!strcasecmp("on", var->value)) {
-		endpoint->stir_shaken = AST_SIP_STIR_SHAKEN_ON;
-	} else {
-		ast_log(LOG_WARNING, "'%s' is not a valid value for option "
-			"'stir_shaken' for endpoint %s\n",
-			var->value, ast_sorcery_object_get_id(endpoint));
-		return -1;
-	}
-
-	return 0;
-}
-
-static const char *stir_shaken_map[] = {
-	[AST_SIP_STIR_SHAKEN_OFF] = "off",
-	[AST_SIP_STIR_SHAKEN_ATTEST] = "attest",
-	[AST_SIP_STIR_SHAKEN_VERIFY] = "verify",
-	[AST_SIP_STIR_SHAKEN_ON] = "on",
-};
-
-static int stir_shaken_to_str(const void *obj, const intptr_t *args, char **buf)
-{
-	const struct ast_sip_endpoint *endpoint = obj;
-	if (ARRAY_IN_BOUNDS(endpoint->stir_shaken, stir_shaken_map)) {
-		*buf = ast_strdup(stir_shaken_map[endpoint->stir_shaken]);
-	}
-	return 0;
-}
-
 static int group_handler(const struct aco_option *opt,
 			 struct ast_variable *var, void *obj)
 {
@@ -1194,6 +1121,165 @@ static int contact_user_to_str(const void *obj, const intptr_t *args, char **buf
 	return 0;
 }
 
+static int call_offer_pref_handler(const struct aco_option *opt,
+	struct ast_variable *var, void *obj)
+{
+	struct ast_sip_endpoint *endpoint = obj;
+	struct ast_flags pref = { 0, };
+	int outgoing = strcmp(var->name, "outgoing_call_offer_pref") == 0;
+
+	int res = ast_sip_call_codec_str_to_pref(&pref, var->value, outgoing);
+	if (res != 0) {
+		return -1;
+	}
+
+	if (outgoing) {
+		endpoint->media.outgoing_call_offer_pref = pref;
+	} else {
+		endpoint->media.incoming_call_offer_pref = pref;
+	}
+
+	return 0;
+}
+
+static int incoming_call_offer_pref_to_str(const void *obj, const intptr_t *args, char **buf)
+{
+	const struct ast_sip_endpoint *endpoint = obj;
+
+	*buf = ast_strdup(ast_sip_call_codec_pref_to_str(endpoint->media.incoming_call_offer_pref));
+	if (!(*buf)) {
+		return -1;
+	}
+
+	return 0;
+}
+
+static int outgoing_call_offer_pref_to_str(const void *obj, const intptr_t *args, char **buf)
+{
+	const struct ast_sip_endpoint *endpoint = obj;
+
+	*buf = ast_strdup(ast_sip_call_codec_pref_to_str(endpoint->media.outgoing_call_offer_pref));
+	if (!(*buf)) {
+		return -1;
+	}
+
+	return 0;
+}
+
+static int codec_prefs_handler(const struct aco_option *opt,
+	struct ast_variable *var, void *obj)
+{
+	struct ast_sip_endpoint *endpoint = obj;
+	struct ast_stream_codec_negotiation_prefs *option_prefs;
+	struct ast_stream_codec_negotiation_prefs prefs;
+	struct ast_str *error_message = ast_str_create(128);
+	enum ast_stream_codec_negotiation_prefs_prefer_values default_prefer;
+	enum ast_stream_codec_negotiation_prefs_operation_values default_operation;
+	int res = 0;
+
+	res = ast_stream_codec_prefs_parse(var->value, &prefs, &error_message);
+	if (res < 0) {
+		ast_log(LOG_ERROR, "Endpoint '%s': %s for option '%s'\n",
+			ast_sorcery_object_get_id(endpoint), ast_str_buffer(error_message), var->name);
+		ast_free(error_message);
+		return -1;
+	}
+	ast_free(error_message);
+
+	if (strcmp(var->name, "codec_prefs_incoming_offer") == 0) {
+		if (prefs.operation == CODEC_NEGOTIATION_OPERATION_UNION) {
+			ast_log(LOG_ERROR, "Endpoint '%s': Codec preference '%s' has invalid value '%s' for option: '%s'",
+				ast_sorcery_object_get_id(endpoint),
+				ast_stream_codec_param_to_str(CODEC_NEGOTIATION_PARAM_OPERATION),
+				ast_stream_codec_operation_to_str(CODEC_NEGOTIATION_OPERATION_UNION),
+				var->name);
+			return -1;
+		}
+		option_prefs = &endpoint->media.codec_prefs_incoming_offer;
+		default_prefer = CODEC_NEGOTIATION_PREFER_PENDING;
+		default_operation = CODEC_NEGOTIATION_OPERATION_INTERSECT;
+	} else if (strcmp(var->name, "codec_prefs_outgoing_offer") == 0) {
+		option_prefs = &endpoint->media.codec_prefs_outgoing_offer;
+		default_prefer = CODEC_NEGOTIATION_PREFER_PENDING;
+		default_operation = CODEC_NEGOTIATION_OPERATION_UNION;
+	} else if (strcmp(var->name, "codec_prefs_incoming_answer") == 0) {
+		option_prefs = &endpoint->media.codec_prefs_incoming_answer;
+		default_prefer = CODEC_NEGOTIATION_PREFER_PENDING;
+		default_operation = CODEC_NEGOTIATION_OPERATION_INTERSECT;
+	} else if (strcmp(var->name, "codec_prefs_outgoing_answer") == 0) {
+		option_prefs = &endpoint->media.codec_prefs_outgoing_answer;
+		default_prefer = CODEC_NEGOTIATION_PREFER_PENDING;
+		default_operation = CODEC_NEGOTIATION_OPERATION_INTERSECT;
+	} else {
+		ast_log(LOG_ERROR, "Endpoint '%s': Unsupported option '%s'\n",
+			ast_sorcery_object_get_id(endpoint),
+			var->name);
+		return -1;
+	}
+
+	if (prefs.prefer == CODEC_NEGOTIATION_PREFER_UNSPECIFIED) {
+		prefs.prefer = default_prefer;
+	}
+
+	if (prefs.operation == CODEC_NEGOTIATION_OPERATION_UNSPECIFIED) {
+		prefs.operation = default_operation;
+	}
+
+	if (prefs.keep == CODEC_NEGOTIATION_KEEP_UNSPECIFIED) {
+		prefs.keep = CODEC_NEGOTIATION_KEEP_ALL;
+	}
+
+	if (prefs.transcode == CODEC_NEGOTIATION_TRANSCODE_UNSPECIFIED) {
+		prefs.transcode = CODEC_NEGOTIATION_TRANSCODE_ALLOW;
+	}
+
+	/* Now that defaults have been applied as needed we apply the full codec
+	 * preference configuration to the option.
+	 */
+	*option_prefs = prefs;
+
+	return 0;
+}
+
+static int codec_prefs_to_str(const struct ast_stream_codec_negotiation_prefs *prefs,
+	const void *obj, const intptr_t *args, char **buf)
+{
+	struct ast_str *codecs = ast_str_create(AST_STREAM_MAX_CODEC_PREFS_LENGTH);
+
+	if (!codecs) {
+		return -1;
+	}
+
+	*buf = ast_strdup(ast_stream_codec_prefs_to_str(prefs, &codecs));
+	ast_free(codecs);
+
+	return 0;
+}
+
+static int incoming_offer_codec_prefs_to_str(const void *obj, const intptr_t *args, char **buf)
+{
+	const struct ast_sip_endpoint *endpoint = obj;
+	return codec_prefs_to_str(&endpoint->media.codec_prefs_incoming_offer, obj, args, buf);
+}
+
+static int outgoing_offer_codec_prefs_to_str(const void *obj, const intptr_t *args, char **buf)
+{
+	const struct ast_sip_endpoint *endpoint = obj;
+	return codec_prefs_to_str(&endpoint->media.codec_prefs_outgoing_offer, obj, args, buf);
+}
+
+static int incoming_answer_codec_prefs_to_str(const void *obj, const intptr_t *args, char **buf)
+{
+	const struct ast_sip_endpoint *endpoint = obj;
+	return codec_prefs_to_str(&endpoint->media.codec_prefs_incoming_answer, obj, args, buf);
+}
+
+static int outgoing_answer_codec_prefs_to_str(const void *obj, const intptr_t *args, char **buf)
+{
+	const struct ast_sip_endpoint *endpoint = obj;
+	return codec_prefs_to_str(&endpoint->media.codec_prefs_outgoing_answer, obj, args, buf);
+}
+
 static void *sip_nat_hook_alloc(const char *name)
 {
 	return ast_sorcery_generic_alloc(sizeof(struct ast_sip_nat_hook), NULL);
@@ -1376,6 +1462,16 @@ static int sip_endpoint_apply_handler(const struct ast_sorcery *sorcery, void *o
 		return -1;
 	}
 
+	if (endpoint->preferred_codec_only) {
+		if (endpoint->media.incoming_call_offer_pref.flags != (AST_SIP_CALL_CODEC_PREF_LOCAL | AST_SIP_CALL_CODEC_PREF_INTERSECT | AST_SIP_CALL_CODEC_PREF_ALL)) {
+			ast_log(LOG_ERROR, "Setting both preferred_codec_only and incoming_call_offer_pref is not supported on endpoint '%s'\n",
+				ast_sorcery_object_get_id(endpoint));
+			return -1;
+		}
+		ast_clear_flag(&endpoint->media.incoming_call_offer_pref, AST_SIP_CALL_CODEC_PREF_ALL);
+		ast_set_flag(&endpoint->media.incoming_call_offer_pref, AST_SIP_CALL_CODEC_PREF_FIRST);
+	}
+
 	endpoint->media.topology = ast_stream_topology_create_from_format_cap(endpoint->media.codecs);
 	if (!endpoint->media.topology) {
 		return -1;
@@ -1405,36 +1501,6 @@ static int sip_endpoint_apply_handler(const struct ast_sorcery *sorcery, void *o
 		}
 	}
 
-	if (!ast_strlen_zero(endpoint->geoloc_incoming_call_profile) ||
-		!ast_strlen_zero(endpoint->geoloc_outgoing_call_profile)) {
-
-		if (!ast_geoloc_is_loaded()) {
-			ast_log(LOG_ERROR, "A geoloc incoming and/or outgoing call_profile was specified on endpoint '%s'"
-				" but res_geolocation is not loaded.\n", ast_sorcery_object_get_id(endpoint));
-			return -1;
-		}
-
-		if (!ast_strlen_zero(endpoint->geoloc_incoming_call_profile)) {
-			struct ast_geoloc_profile *profile = ast_geoloc_get_profile(endpoint->geoloc_incoming_call_profile);
-			if (!profile) {
-				ast_log(LOG_ERROR, "geoloc_incoming_call_profile '%s' on endpoint '%s' doesn't exist\n",
-					endpoint->geoloc_incoming_call_profile, ast_sorcery_object_get_id(endpoint));
-				return -1;
-			}
-			ao2_cleanup(profile);
-		}
-
-		if (!ast_strlen_zero(endpoint->geoloc_outgoing_call_profile)) {
-			struct ast_geoloc_profile *profile = ast_geoloc_get_profile(endpoint->geoloc_outgoing_call_profile);
-			if (!profile) {
-				ast_log(LOG_ERROR, "geoloc_outgoing_call_profile '%s' on endpoint '%s' doesn't exist\n",
-					endpoint->geoloc_outgoing_call_profile, ast_sorcery_object_get_id(endpoint));
-				return -1;
-			}
-			ao2_cleanup(profile);
-		}
-	}
-
 	return 0;
 }
 
@@ -1493,7 +1559,7 @@ static int active_channels_to_str_cb(void *object, void *arg, int flags)
 {
 	const struct ast_channel_snapshot *snapshot = object;
 	struct ast_str **buf = arg;
-	ast_str_append(buf, 0, "%s,", snapshot->name);
+	ast_str_append(buf, 0, "%s,", snapshot->base->name);
 	return 0;
 }
 
@@ -1971,8 +2037,8 @@ int ast_res_pjsip_initialize_configuration(void)
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "identify_by", "username,ip", ident_handler, ident_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "direct_media", "yes", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.direct_media.enabled));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "direct_media_method", "invite", direct_media_method_handler, direct_media_method_to_str, NULL, 0, 0);
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "trust_connected_line", "yes", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, trust_connected_line));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "send_connected_line", "yes", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, send_connected_line));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "trust_connected_line", "yes", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, id.trust_connected_line));
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "send_connected_line", "yes", OPT_YESNO_T, 1, FLDSET(struct ast_sip_endpoint, id.send_connected_line));
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "connected_line_method", "invite", connected_line_method_handler, connected_line_method_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "direct_media_glare_mitigation", "none", direct_media_glare_mitigation_handler, direct_media_glare_mitigation_to_str, NULL, 0, 0);
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "disable_direct_media_on_nat", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.direct_media.disable_on_nat));
@@ -2011,7 +2077,6 @@ int ast_res_pjsip_initialize_configuration(void)
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "fax_detect_timeout", "0", OPT_UINT_T, 0, FLDSET(struct ast_sip_endpoint, faxdetect_timeout));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "t38_udptl_nat", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.t38.nat));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "t38_udptl_ipv6", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.t38.ipv6));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "t38_bind_udptl_to_media_address", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.t38.bind_udptl_to_media_address));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "tone_zone", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, zone));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "language", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, language));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "record_on_feature", "automixmon", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, info.recording.onfeature));
@@ -2071,13 +2136,24 @@ int ast_res_pjsip_initialize_configuration(void)
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "accept_multiple_sdp_answers", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, media.rtp.accept_multiple_sdp_answers));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "suppress_q850_reason_headers", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, suppress_q850_reason_headers));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "ignore_183_without_sdp", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, ignore_183_without_sdp));
-	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "stir_shaken", "off", stir_shaken_handler, stir_shaken_to_str, NULL, 0, 0);
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "stir_shaken_profile", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, stir_shaken_profile));
+	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "incoming_call_offer_pref", "local",
+		call_offer_pref_handler, incoming_call_offer_pref_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "outgoing_call_offer_pref", "remote_merge",
+		call_offer_pref_handler, outgoing_call_offer_pref_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "codec_prefs_incoming_offer",
+		"prefer: pending, operation: intersect, keep: all, transcode: allow",
+		codec_prefs_handler, incoming_offer_codec_prefs_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "codec_prefs_outgoing_offer",
+		"prefer: pending, operation: union, keep: all, transcode: allow",
+		codec_prefs_handler, outgoing_offer_codec_prefs_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "codec_prefs_incoming_answer",
+		"prefer: pending, operation: intersect, keep: all",
+		codec_prefs_handler, incoming_answer_codec_prefs_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "codec_prefs_outgoing_answer",
+		"prefer: pending, operation: intersect, keep: all",
+		codec_prefs_handler, outgoing_answer_codec_prefs_to_str, NULL, 0, 0);
+	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "stir_shaken", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, stir_shaken));
 	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "allow_unauthenticated_options", "no", OPT_BOOL_T, 1, FLDSET(struct ast_sip_endpoint, allow_unauthenticated_options));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "geoloc_incoming_call_profile", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, geoloc_incoming_call_profile));
-	ast_sorcery_object_field_register(sip_sorcery, "endpoint", "geoloc_outgoing_call_profile", "", OPT_STRINGFIELD_T, 0, STRFLDSET(struct ast_sip_endpoint, geoloc_outgoing_call_profile));
-	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "security_mechanisms", "", security_mechanism_handler, NULL, NULL, 0, 0);
-	ast_sorcery_object_field_register_custom(sip_sorcery, "endpoint", "security_negotiation", "no", security_negotiation_handler, NULL, NULL, 0, 0);
 
 	if (ast_sip_initialize_sorcery_transport()) {
 		ast_log(LOG_ERROR, "Failed to register SIP transport support with sorcery\n");
@@ -2131,6 +2207,8 @@ int ast_res_pjsip_initialize_configuration(void)
 
 	load_all_endpoints();
 
+	ast_sip_location_prune_boot_contacts();
+
 	acl_change_sub = stasis_subscribe(ast_security_topic(), acl_change_stasis_cb, NULL);
 	stasis_subscription_accept_message_type(acl_change_sub, ast_named_acl_change_type());
 	stasis_subscription_set_filter(acl_change_sub, STASIS_SUBSCRIPTION_FILTER_SELECTIVE);
@@ -2236,12 +2314,6 @@ void *ast_sip_endpoint_alloc(const char *name)
 		return NULL;
 	}
 
-	if (ast_string_field_init_extended(endpoint, geoloc_incoming_call_profile) ||
-		ast_string_field_init_extended(endpoint, geoloc_outgoing_call_profile)) {
-		ao2_cleanup(endpoint);
-		return NULL;
-	}
-
 	if (!(endpoint->media.codecs = ast_format_cap_alloc(AST_FORMAT_CAP_FLAG_DEFAULT))) {
 		ao2_cleanup(endpoint);
 		return NULL;
@@ -2310,25 +2382,6 @@ void ast_sip_cleanup_auths(struct ast_sip_auth *auths[], size_t num_auths)
 	}
 }
 
-int ast_sip_retrieve_auths_vector(const struct ast_sip_auth_vector *auth_ids,
-	struct ast_sip_auth_objects_vector *auth_objects)
-{
-	int i;
-
-	for (i = 0; i < AST_VECTOR_SIZE(auth_ids); ++i) {
-		/* Using AST_VECTOR_GET is safe since the vector is immutable */
-		const char *name = AST_VECTOR_GET(auth_ids, i);
-		struct ast_sip_auth *auth_object = ast_sorcery_retrieve_by_id(ast_sip_get_sorcery(), SIP_SORCERY_AUTH_TYPE, name);
-		if (!auth_object) {
-			ast_log(LOG_WARNING, "Auth object '%s' could not be found\n", name);
-		} else {
-			AST_VECTOR_APPEND(auth_objects, auth_object);
-		}
-	}
-
-	return AST_VECTOR_SIZE(auth_objects) == AST_VECTOR_SIZE(auth_ids) ? 0 : -1;
-}
-
 struct ast_sorcery *ast_sip_get_sorcery(void)
 {
 	return sip_sorcery;
